@@ -1,7 +1,7 @@
 __title__ = 'qq'
 __author__ = 'Foxwhite'
 __license__ = 'MIT'
-__version__ = '0.0.5'
+__version__ = '0.1.0'
 
 __path__ = __import__('pkgutil').extend_path(__path__, __name__)
 
@@ -10,5 +10,5 @@ import logging
 from .client import *
 from .guild import *
 
-
-logging.getLogger(__name__).addHandler(logging.NullHandler())
+logging.basicConfig()
+logging.root.setLevel(logging.NOTSET)
