@@ -5,16 +5,19 @@ import logging
 import sys
 import weakref
 from types import TracebackType
-from typing import ClassVar, Any, Optional, Sequence, Iterable, Dict, Union, TypeVar, Type, Coroutine, List
+from typing import ClassVar, Any, Optional, Sequence, Iterable, Dict, Union, TypeVar, Type, Coroutine, List, Tuple
 from urllib.parse import quote as _uriquote
 import aiohttp
+import requests
 
 from . import __version__, utils, role
-from .error import HTTPException, Forbidden, NotFound, QQServerError, LoginFailure
+from .error import HTTPException, Forbidden, NotFound, QQServerError, LoginFailure, GatewayNotFound
 from .gateway import QQClientWebSocketResponse
 from .message import Message
 from .types import user, guild
 from .utils import MISSING
+from .types.channel import Channel as ChannelPayload
+from .types.role import Role as RolePayload
 
 T = TypeVar('T')
 BE = TypeVar('BE', bound=BaseException)
@@ -219,7 +222,7 @@ class HTTPClient:
 
     def get_guilds(
             self,
-            limit: int,
+            limit: int = 100,
             before: Optional[str] = None,
             after: Optional[str] = None,
     ) -> Response[List[guild.Guild]]:
@@ -234,68 +237,123 @@ class HTTPClient:
 
         return self.request(Route('GET', '/users/@me/guilds'), params=params)
 
+    def sync_guild_channels_roles(self, guild_id: int) -> Tuple[List[ChannelPayload], List[RolePayload]]:
+        headers: Dict[str, str] = {
+            'User-Agent': self.user_agent,
+        }
+        if self.token is not None:
+            headers['Authorization'] = 'Bot ' + self.token
+        rsp = requests.get(f'https://api.sgroup.qq.com/guilds/{guild_id}/channels', headers=headers)
+        rsp2 = requests.get(f'https://api.sgroup.qq.com/guilds/{guild_id}/roles', headers=headers)
+        return rsp.json(), rsp2.json()['roles']
+
     def get_guild(self, guild_id: int) -> Response[guild.Guild]:
         return self.request(Route('GET', '/guilds/{guild_id}', guild_id=guild_id))
 
     def get_guild_channels(self, guild_id: int) -> Response[guild.Guild]:
         return self.request(Route('GET', '/guilds/{guild_id}/channels', guild_id=guild_id))
 
-    # def get_message(self, channel_id: int, message_id: int) -> Response[Message]:
-    #     r = Route('GET', '/channels/{channel_id}/messages/{message_id}', channel_id=channel_id, message_id=message_id)
-    #     return self.request(r)
-    #
-    # # 身份组管理
-    #
-    # def get_roles(self, guild_id: int) -> Response[List[role.Role]]:
-    #     return self.request(Route('GET', '/guilds/{guild_id}/roles', guild_id=guild_id))
-    #
-    # def edit_role(
-    #         self, guild_id: int, role_id: int, *, reason: Optional[str] = None, **fields: Any
-    # ) -> Response[role.Role]:
-    #     r = Route('PATCH', '/guilds/{guild_id}/roles/{role_id}', guild_id=guild_id, role_id=role_id)
-    #     valid_keys = ('name', 'color', 'hoist')
-    #     payload = {"info": {k: v for k, v in fields.items() if k in valid_keys}}
-    #     return self.request(r, json=payload, reason=reason)
-    #
-    # def delete_role(self, guild_id: int, role_id: int, *, reason: Optional[str] = None) -> Response[None]:
-    #     r = Route('DELETE', '/guilds/{guild_id}/roles/{role_id}', guild_id=guild_id, role_id=role_id)
-    #     return self.request(r, reason=reason)
-    #
-    # def create_role(self, guild_id: int, *, reason: Optional[str] = None, **fields: Any) -> Response[role.Role]:
-    #     r = Route('POST', '/guilds/{guild_id}/roles', guild_id=guild_id)
-    #     return self.request(r, json=fields, reason=reason)
-    #
-    # def add_role(
-    #         self, guild_id: int, user_id: int, role_id: int, *, reason: Optional[str] = None
-    # ) -> Response[None]:
-    #     r = Route(
-    #         'PUT',
-    #         '/guilds/{guild_id}/members/{user_id}/roles/{role_id}',
-    #         guild_id=guild_id,
-    #         user_id=user_id,
-    #         role_id=role_id,
-    #     )
-    #     return self.request(r, reason=reason)
-    #
-    # def remove_role(
-    #         self, guild_id: str, user_id: str, role_id: str, *, reason: Optional[str] = None
-    # ) -> Response[None]:
-    #     r = Route(
-    #         'DELETE',
-    #         '/guilds/{guild_id}/members/{user_id}/roles/{role_id}',
-    #         guild_id=guild_id,
-    #         user_id=user_id,
-    #         role_id=role_id,
-    #     )
-    #     return self.request(r, reason=reason)
-    #
-    # async def get_from_cdn(self, url: str) -> bytes:
-    #     async with self.__session.get(url) as resp:
-    #         if resp.status == 200:
-    #             return await resp.read()
-    #         elif resp.status == 404:
-    #             raise NotFound(resp, 'asset not found')
-    #         elif resp.status == 403:
-    #             raise Forbidden(resp, 'cannot retrieve asset')
-    #         else:
-    #             raise HTTPException(resp, 'failed to get asset')
+    def get_message(self, channel_id: int, message_id: int) -> Response[Message]:
+        r = Route('GET', '/channels/{channel_id}/messages/{message_id}', channel_id=channel_id, message_id=message_id)
+        return self.request(r)
+
+    # 身份组管理
+
+    def get_roles(self, guild_id: int) -> Response[List[role.Role]]:
+        return self.request(Route('GET', '/guilds/{guild_id}/roles', guild_id=guild_id))
+
+    def edit_role(
+            self, guild_id: int, role_id: int, *, reason: Optional[str] = None, **fields: Any
+    ) -> Response[role.Role]:
+        r = Route('PATCH', '/guilds/{guild_id}/roles/{role_id}', guild_id=guild_id, role_id=role_id)
+        valid_keys = ('name', 'color', 'hoist')
+        payload = {"info": {k: v for k, v in fields.items() if k in valid_keys}}
+        return self.request(r, json=payload, reason=reason)
+
+    def delete_role(self, guild_id: int, role_id: int, *, reason: Optional[str] = None) -> Response[None]:
+        r = Route('DELETE', '/guilds/{guild_id}/roles/{role_id}', guild_id=guild_id, role_id=role_id)
+        return self.request(r, reason=reason)
+
+    def create_role(self, guild_id: int, *, reason: Optional[str] = None, **fields: Any) -> Response[role.Role]:
+        r = Route('POST', '/guilds/{guild_id}/roles', guild_id=guild_id)
+        return self.request(r, json=fields, reason=reason)
+
+    def add_role(
+            self, guild_id: int, user_id: int, role_id: int, *, reason: Optional[str] = None
+    ) -> Response[None]:
+        r = Route(
+            'PUT',
+            '/guilds/{guild_id}/members/{user_id}/roles/{role_id}',
+            guild_id=guild_id,
+            user_id=user_id,
+            role_id=role_id,
+        )
+        return self.request(r, reason=reason)
+
+    def remove_role(
+            self, guild_id: str, user_id: str, role_id: str, *, reason: Optional[str] = None
+    ) -> Response[None]:
+        r = Route(
+            'DELETE',
+            '/guilds/{guild_id}/members/{user_id}/roles/{role_id}',
+            guild_id=guild_id,
+            user_id=user_id,
+            role_id=role_id,
+        )
+        return self.request(r, reason=reason)
+
+    async def get_from_cdn(self, url: str) -> bytes:
+        async with self.__session.get(url) as resp:
+            if resp.status == 200:
+                return await resp.read()
+            elif resp.status == 404:
+                raise NotFound(resp, 'asset not found')
+            elif resp.status == 403:
+                raise Forbidden(resp, 'cannot retrieve asset')
+            else:
+                raise HTTPException(resp, 'failed to get asset')
+
+    async def get_gateway(self, *, encoding: str = 'json', zlib: bool = True) -> str:
+        try:
+            data = await self.request(Route('GET', '/gateway'))
+        except HTTPException as exc:
+            raise GatewayNotFound() from exc
+        return data['url']
+
+    async def get_bot_gateway(self, *, encoding: str = 'json', zlib: bool = True) -> Tuple[int, str]:
+        try:
+            data = await self.request(Route('GET', '/gateway/bot'))
+        except HTTPException as exc:
+            raise GatewayNotFound() from exc
+
+        if zlib:
+            value = '{0}?encoding={1}&v=9&compress=zlib-stream'
+        else:
+            value = '{0}?encoding={1}&v=9'
+        return data['shards'], value.format(data['url'], encoding)
+
+    def recreate(self) -> None:
+        if self.__session.closed:
+            self.__session = aiohttp.ClientSession(
+                connector=self.connector, ws_response_class=QQClientWebSocketResponse
+            )
+
+    async def close(self) -> None:
+        if self.__session:
+            await self.__session.close()
+
+    async def ws_connect(self, url: str, *, compress: int = 0) -> Any:
+        kwargs = {
+            'proxy_auth': self.proxy_auth,
+            'proxy': self.proxy,
+            'max_msg_size': 0,
+            'timeout': 30.0,
+            'autoclose': False,
+            'headers': {
+                'User-Agent': self.user_agent,
+            },
+            'compress': compress,
+        }
+
+        return await self.__session.ws_connect(url, **kwargs)
+
