@@ -1,8 +1,10 @@
 import array
+import asyncio
 import datetime
 import json
 import re
 from bisect import bisect_left
+from operator import attrgetter
 from typing import Any, Callable, TypeVar, overload, Optional, Iterable, List, TYPE_CHECKING, Generic, Type
 from inspect import isawaitable as _isawaitable, signature as _signature
 
@@ -147,3 +149,41 @@ def cached_slot_property(name: str) -> Callable[[Callable[[T], T_co]], CachedSlo
 
 def escape_mentions(text: str) -> str:
     return re.sub(r'@(everyone|here|[!&]?[0-9]{17,20})', '@\u200b\\1', text)
+
+
+def find(predicate: Callable[[T], Any], seq: Iterable[T]) -> Optional[T]:
+    for element in seq:
+        if predicate(element):
+            return element
+    return None
+
+
+def get(iterable: Iterable[T], **attrs: Any) -> Optional[T]:
+    _all = all
+    attrget = attrgetter
+
+    # Special case the single element call
+    if len(attrs) == 1:
+        k, v = attrs.popitem()
+        pred = attrget(k.replace('__', '.'))
+        for elem in iterable:
+            if pred(elem) == v:
+                return elem
+        return None
+
+    converted = [(attrget(attr.replace('__', '.')), value) for attr, value in attrs.items()]
+
+    for elem in iterable:
+        if _all(pred(elem) == value for pred, value in converted):
+            return elem
+    return None
+
+
+async def sane_wait_for(futures, *, timeout):
+    ensured = [asyncio.ensure_future(fut) for fut in futures]
+    done, pending = await asyncio.wait(ensured, timeout=timeout, return_when=asyncio.ALL_COMPLETED)
+
+    if len(pending) != 0:
+        raise asyncio.TimeoutError()
+
+    return done
